@@ -35,6 +35,14 @@ Transaction::Transaction(QObject *parent, AbstractResource *resource,
 {
 }
 
+Transaction::~Transaction()
+{
+    TransactionModel* tm = TransactionModel::global();
+    if (tm->indexOf(this).isValid()) {
+        tm->removeTransaction(this);
+    }
+}
+
 AbstractResource *Transaction::resource() const
 {
     return m_resource;
@@ -94,3 +102,7 @@ void Transaction::setProgress(int progress)
     }
 }
 
+bool Transaction::isActive() const
+{
+    return m_status == DownloadingStatus || m_status == CommittingStatus;
+}
