@@ -77,6 +77,8 @@ public:
     Transaction(QObject *parent, AbstractResource *resource,
                  Transaction::Role role, const AddonList &addons = {});
 
+    ~Transaction() override;
+
     /**
      * @returns the AbstractResource which this transaction works with
      */
@@ -160,6 +162,8 @@ Q_SIGNALS:
      * @sa proceed(), cancel()
      */
     void proceedRequest(const QString &title, const QString &description);
+
+    void passiveMessage(const QString &message);
 };
 
 #endif // TRANSACTION_H
