@@ -45,24 +45,6 @@
 
 MUON_BACKEND_PLUGIN(KNSBackend)
 
-QSharedPointer<Attica::ProviderManager> KNSBackend::m_atticaManager;
-
-void KNSBackend::initManager(const QUrl& entry)
-{
-    bool loadNeeded = false;
-    if(!m_atticaManager) {
-        m_atticaManager = QSharedPointer<Attica::ProviderManager>(new Attica::ProviderManager);
-        loadNeeded = true;
-    }
-    if(!m_atticaManager->defaultProviderFiles().contains(entry)) {
-        m_atticaManager->addProviderFileToDefaultProviders(entry);
-        loadNeeded = true;
-    }
-
-    if(loadNeeded)
-        m_atticaManager->loadDefaultProviders();
-}
-
 KNSBackend::KNSBackend(QObject* parent)
     : AbstractResourcesBackend(parent)
     , m_fetching(false)
@@ -74,9 +56,9 @@ KNSBackend::KNSBackend(QObject* parent)
 
 KNSBackend::~KNSBackend() = default;
 
-void KNSBackend::markInvalid()
+void KNSBackend::markInvalid(const QString &message)
 {
-    qWarning() << "invalid kns backend!";
+    qWarning() << "invalid kns backend!" << m_name << "because:" << message;
     m_isValid = false;
     setFetching(false);
 }
@@ -87,7 +69,8 @@ void KNSBackend::setMetaData(const QString& path)
     KConfigGroup service = cfg.group("Desktop Entry");
 
     m_iconName = service.readEntry("Icon", QString());
-    QString knsrc = service.readEntry("X-Muon-Arguments", QString());
+
+    const QString knsrc = service.readEntry("X-Muon-Arguments", QString());
     m_name = QStandardPaths::locate(QStandardPaths::GenericConfigLocation, knsrc);
     if (m_name.isEmpty()) {
         QString p = QFileInfo(path).dir().filePath(knsrc);
@@ -97,38 +80,29 @@ void KNSBackend::setMetaData(const QString& path)
     }
 
     if (m_name.isEmpty()) {
-        markInvalid();
-        qWarning() << "Couldn't find knsrc file" << knsrc;
-        return;
-    }
-    KConfig conf(m_name);
-    KConfigGroup group;
-
-    if (conf.hasGroup("KNewStuff3"))
-        group = conf.group("KNewStuff3");
-
-    if (!group.isValid()) {
-        markInvalid();
-        qWarning() << "Config group not found! Check your KNS3 installation.";
+        markInvalid(QStringLiteral("Couldn't find knsrc file: ") + knsrc);
         return;
     }
 
-    QStringList cats = group.readEntry("Categories", QStringList());
-    initManager(QUrl(group.readEntry("ProvidersUrl", QString())));
-    connect(m_atticaManager.data(), &Attica::ProviderManager::defaultProvidersLoaded, this, &KNSBackend::startFetchingCategories);
-
-    foreach(const QString& c, cats) {
-        m_categories.insert(c, Attica::Category());
+    const KConfig conf(m_name);
+    if (!conf.hasGroup("KNewStuff3")) {
+        markInvalid(QStringLiteral("Config group not found! Check your KNS3 installation."));
+        return;
     }
 
+    const KConfigGroup group = conf.group("KNewStuff3");
+    m_extends = group.readEntry("Extends", QStringList());
+    m_reviews->setProviderUrl(QUrl(group.readEntry("ProvidersUrl", QString())));
+
+    setFetching(true);
+
     m_manager = new KNS3::DownloadManager(m_name, this);
+    connect(m_manager, &KNS3::DownloadManager::errorFound, this, &KNSBackend::markInvalid);
     connect(m_manager, &KNS3::DownloadManager::searchResult, this, &KNSBackend::receivedEntries);
     connect(m_manager, &KNS3::DownloadManager::entryStatusChanged, this, &KNSBackend::statusChanged);
 
-    //otherwise this will be executed when defaultProvidersLoaded is emitted
-    if (!m_atticaManager->providers().isEmpty()) {
-        startFetchingCategories();
-    }
+    m_page = 0;
+    m_manager->search(m_page);
 }
 
 void KNSBackend::setFetching(bool f)
@@ -144,97 +118,17 @@ bool KNSBackend::isValid() const
     return m_isValid;
 }
 
-void KNSBackend::startFetchingCategories()
-{
-    if (m_atticaManager->providers().isEmpty()) {
-        qWarning() << "no providers for" << m_name;
-        markInvalid();
-        return;
-    }
-
-    setFetching(true);
-    m_provider = m_atticaManager->providers().first();
-
-    Attica::ListJob<Attica::Category>* job = m_provider.requestCategories();
-    connect(job, &Attica::GetJob::finished, this, &KNSBackend::categoriesLoaded);
-    job->start();
-}
-
-void KNSBackend::categoriesLoaded(Attica::BaseJob* job)
-{
-    if(job->metadata().error() != Attica::Metadata::NoError) {
-        qWarning() << "Network error";
-        setFetching(false);
-        return;
-    }
-    Attica::ListJob<Attica::Category>* j = static_cast<Attica::ListJob<Attica::Category>*>(job);
-    Attica::Category::List categoryList = j->itemList();
-
-    foreach(const Attica::Category& category, categoryList) {
-        if (m_categories.contains(category.name())) {
-//             qDebug() << "Adding category: " << category.name();
-            m_categories[category.name()] = category;
-        }
-    }
-    
-    // Remove categories for which we got no matching category from the provider.
-    // Otherwise we'll fetch an empty category specificier which can return
-    // everything and the kitchen sink if the remote provider feels like it.
-    for (auto it = m_categories.begin(); it != m_categories.end();) {
-        if (!it.value().isValid()) {
-            qWarning() << "Found invalid category" << it.key();
-            it = m_categories.erase(it);
-        } else
-            ++it;
-    }
-    if (m_categories.isEmpty()) {
-        markInvalid();
-        return;
-    }
-
-    Attica::ListJob<Attica::Content>* jj =
-        m_provider.searchContents(m_categories.values(), QString(), Attica::Provider::Alphabetical, m_page, 100);
-    connect(jj, &Attica::GetJob::finished, this, &KNSBackend::receivedContents);
-    jj->start();
-}
-
-void KNSBackend::receivedContents(Attica::BaseJob* job)
-{
-    if(job->metadata().error() != Attica::Metadata::NoError) {
-        qWarning() << "Network error";
-        setFetching(false);
-        return;
-    }
-    Attica::ListJob<Attica::Content>* listJob = static_cast<Attica::ListJob<Attica::Content>*>(job);
-    Attica::Content::List contents = listJob->itemList();
-    
-    if(contents.isEmpty()) {
-        m_page = 0;
-        m_manager->search();
-        return;
-    }
-    QString filename = QFileInfo(m_name).fileName();
-    foreach(const Attica::Content& c, contents) {
-        KNSResource* r = new KNSResource(c, filename, this);
-        m_resourcesByName.insert(c.id(), r);
-        connect(r, &KNSResource::stateChanged, this, &KNSBackend::updatesCountChanged);
-    }
-    m_page++;
-    Attica::ListJob<Attica::Content>* jj =
-        m_provider.searchContents(m_categories.values(), QString(), Attica::Provider::Alphabetical, m_page, 100);
-    connect(jj, &Attica::GetJob::finished, this, &KNSBackend::receivedContents);
-    jj->start();
-}
-
 void KNSBackend::receivedEntries(const KNS3::Entry::List& entries)
 {
     if(entries.isEmpty()) {
         setFetching(false);
         return;
     }
-    
+
+    const QString filename = QFileInfo(m_name).fileName();
     foreach(const KNS3::Entry& entry, entries) {
-        statusChanged(entry);
+        KNSResource* r = new KNSResource(entry, filename, this);
+        m_resourcesByName.insert(entry.id(), r);
     }
     ++m_page;
     m_manager->search(m_page);
@@ -249,33 +143,35 @@ void KNSBackend::statusChanged(const KNS3::Entry& entry)
         qWarning() << "unknown entry changed" << entry.id() << entry.name();
 }
 
-void KNSBackend::cancelTransaction(AbstractResource* app)
+class LambdaTransaction : public Transaction
 {
-    Q_UNUSED(app)
+public:
+    LambdaTransaction(QObject* parent, AbstractResource* res, Transaction::Role role)
+        : Transaction(parent, res, role)
+    {
+        setCancellable(false);
+        TransactionModel::global()->addTransaction(this);
+    }
 
-    qWarning("KNS transaction canceling unsupported");
-}
+    ~LambdaTransaction() override {
+        TransactionModel::global()->removeTransaction(this);
+    }
+
+    void cancel() override {}
+};
 
 void KNSBackend::removeApplication(AbstractResource* app)
 {
-    Transaction* t = new Transaction(this, app, Transaction::RemoveRole);
-    TransactionModel *transModel = TransactionModel::global();
-    transModel->addTransaction(t);
+    QScopedPointer<Transaction> t(new LambdaTransaction(this, app, Transaction::RemoveRole));
     KNSResource* r = qobject_cast<KNSResource*>(app);
-    Q_ASSERT(r->entry());
-    m_manager->uninstallEntry(*r->entry());
-    transModel->removeTransaction(t);
+    m_manager->uninstallEntry(r->entry());
 }
 
 void KNSBackend::installApplication(AbstractResource* app)
 {
-    Transaction* t = new Transaction(this, app, Transaction::InstallRole);
-    TransactionModel *transModel = TransactionModel::global();
-    transModel->addTransaction(t);
+    QScopedPointer<Transaction> t(new LambdaTransaction(this, app, Transaction::InstallRole));
     KNSResource* r = qobject_cast<KNSResource*>(app);
-    Q_ASSERT(r->entry());
-    m_manager->installEntry(*r->entry());
-    transModel->removeTransaction(t);
+    m_manager->installEntry(r->entry());
 }
 
 void KNSBackend::installApplication(AbstractResource* app, const AddonList& /*addons*/)
@@ -290,22 +186,7 @@ AbstractResource* KNSBackend::resourceByPackageName(const QString& name) const
 
 int KNSBackend::updatesCount() const
 {
-    int ret = 0;
-    foreach(AbstractResource* r, m_resourcesByName) {
-        if(r->state()==AbstractResource::Upgradeable)
-            ++ret;
-    }
-    return ret;
-}
-
-QList<AbstractResource*> KNSBackend::upgradeablePackages() const
-{
-    QList<AbstractResource*> ret;
-    foreach(AbstractResource* r, m_resourcesByName) {
-        if(r->state()==AbstractResource::Upgradeable)
-            ret+=r;
-    }
-    return ret;
+    return m_updater->updatesCount();
 }
 
 AbstractReviewsBackend* KNSBackend::reviewsBackend() const
@@ -323,9 +204,9 @@ QList<AbstractResource*> KNSBackend::searchPackageName(const QString& searchText
     return ret;
 }
 
-QVector< AbstractResource* > KNSBackend::allResources() const
+QVector<AbstractResource*> KNSBackend::allResources() const
 {
-    return m_resourcesByName.values().toVector();
+    return containerValues<QVector<AbstractResource*>>(m_resourcesByName);
 }
 
 bool KNSBackend::isFetching() const
