@@ -63,6 +63,7 @@ QVariant AppPackageKitResource::icon() const
     if (icons.isEmpty()) {
         ret = QIcon::fromTheme(QStringLiteral("package-x-generic"));
     } else foreach(const AppStream::Icon &icon, icons) {
+        QStringList stock;
         switch(icon.kind()) {
             case AppStream::Icon::KindLocal:
                 ret.addFile(icon.url().toLocalFile(), icon.size());
@@ -70,9 +71,15 @@ QVariant AppPackageKitResource::icon() const
             case AppStream::Icon::KindCached:
                 ret.addFile(icon.url().toLocalFile(), icon.size());
                 break;
+            case AppStream::Icon::KindStock:
+                stock += icon.name();
+                break;
             default:
                 break;
         }
+        if (ret.isNull() && !stock.isEmpty()) {
+            ret = QIcon::fromTheme(stock.first());
+        }
     }
 #else
     const auto icons = m_appdata.iconUrls();
